rem **********************************************************************
rem * actiscsi.vbs                                      Version 1.2.0.1
rem * This VB script file along with the chkiscsi.exe utility is used to
rem * activate host system components that rely on iSCSI devices,
rem * attached through Cisco SN 5400 Series system equipment, to be
rem * present before starting.  Because the iSCSI driver does not log in to
rem * the Cisco SN 5400 Series system equipment until after most of
rem * the Windows OS has already been activated, some services and
rem * components do not initialize properly.  Follow the instructions
rem * below to install the script and utility onto the system to
rem * automatically activate the specified options.
rem *
rem * NOTE: Except for the disk sharing services, ensure that any
rem *       services expected to be started by this script are set to
rem *       manual start using the Windows service utility.
rem *
rem * Place this file in the %SYSTEMROOT%\system32 directory along with
rem * the chkiscsi.exe file.
rem * 
rem * To use the Activate Dynamic Disks option (-d), the Diskpart utility 
rem * will need to be installed into the %SYSTEMROOT%\system32 
rem * directory.  The Diskpart utility can be downloaded from Microsoft 
rem * at this URL:
rem * http://www.microsoft.com/windows2000/techinfo/reskit
rem *      /tools/new/diskpart-o.asp
rem * 
rem * Schedule actiscsi.vbs as a task to run once at boot
rem * time with the following steps:
rem *  - For Windows 2000, open the control panel and click on 
rem *    "Scheduled Tasks".
rem *  - For Windows NT (with Internet Explorer 5.1 or higher), open
rem *    "My Computer" and double click on "Scheduled Tasks".
rem *  - Double click on "Add Scheduled Task" to bring up the wizard.
rem *  - Click Next.
rem *  - Click the Browse button to find and select this file.
rem *  - Click Next.
rem *  - Click the button for "When my computer starts".
rem *  - Click Next to set the privelege level.  The selected user should
rem *    have administrator priveleges.  Enter this user's password.
rem *  - Click Next.
rem *  - Make sure "Open advanced properties..." is checked
rem *  - Click Finish.
rem *  - In the Task tab, place "cmd /c cscript " in front of "actiscsi.vbs" and
rem *    enter the command line arguments to the actiscsi.vbs
rem *    command using this syntax:
rem *      Usage:  actiscsi.vbs -<Version> -<Os> [-e] [-o] [-s] [-t time] [-<Disktype>]
rem *        Version:
rem *            1    Version 1 iSCSI drivers
rem *            2    Version 2 iSCSI drivers
rem *            3    Version 3 iSCSI drivers
rem *        Os:
rem *            n    Windows NT
rem *            w    Windows 2000
rem *        -e    Start Exchange services
rem *        -o    Start Oracle services
rem *        -q    Start MS SQL services 
rem *        -s    Reestablish disk shares
rem *        -t    Delay "time" in seconds to wait after detecting target logins
rem *                   (1 to 120) seconds. Default = 20 seconds
rem *        Disktype:
rem *            d    Activate Dynamic Disks
rem *            v    Import Veritas VM (Version 3.0) disk groups
rem *            v3   Import Veritas VM (Version 3.1) disk groups
rem *     (e.g  "cmd /c cscript actiscsi.vbs -1 -w -o -s" will start Oracle services
rem *           and enable disk shares on iSCSI devices for the
rem *           Windows 2000 iSCSI driver version 1.x.)
rem *  - Click Ok.
rem **********************************************************************

On Error Resume Next

Dim WshShell, fso, f, retVal, iFail, impPos, dgName, cluPos
Dim rdStr, splitStr, impStr, runStr, unkStr, cluStr, tempDir
Dim argCount, echoStr
Dim doV, doO, doQ, doS, doD, doE, ver, Os
Dim diskPartCmds
Dim StartDelay
Dim tryCount
Dim nameStr, dgtypeStr, dgidStr, statStr
Dim strLen

Set WshShell = WScript.CreateObject("WScript.Shell")
Set fso = CreateObject("Scripting.FileSystemObject")

tempDir = WshShell.ExpandEnvironmentStrings("%TEMP%")

iFail = 0
impStr = "Deported"
unkStr = "Unknown DG"
cluStr = "Cluster"
nameStr = "Name"
dgtypeStr = "Disk Group Type"
dgidStr = "DiskGroup ID"
statStr = "Status"
argCount = 0
StartDelay = 20000
Os = "z" 
doO = -1
doQ = -1
doS = -1
doD = -1
doE = -1
doV = -1
ver = -1


rem **********************************************************************
rem * Parse Command Line
rem **********************************************************************
Dim tempArg
' Loop through arguments
While ArgCount < Wscript.Arguments.Count
  
  ' Determine switches used
  Select Case Wscript.Arguments(ArgCount)

	Case "-e":
		If doE > -1 Then
			Wscript.Echo "-e"
			Call UsageMsg
		End If
		doE = 1

	Case "-t":
	    tempArg = Wscript.Arguments(ArgCount + 1)
		If IsNumeric(tempArg) Then
		    ArgCount = ArgCount + 1
			If tempArg >= 1 And tempArg <= 120 Then
			    StartDelay = tempArg * 1000
			Else
			    Wscript.Echo "-t"
			    Call UsageMsg
			End If			    
		End If

	Case "-o":
		If doO > -1 Then
			Wscript.Echo "-o"
			Call UsageMsg
		End If
		doO = 1

	Case "-q":
		If doQ > -1 Then
			Wscript.Echo "-q"
			Call UsageMsg
		End If
		doQ = 1
		
	Case "-s":
		If doS > -1 Then
			Wscript.Echo "-s"
			Call UsageMsg
		End If
		doS = 1

	Case "-d":
		If doD > -1 Then
			Call UsageMsg
		End If
		If doV > -1 Then 'Can't specify both -d and -v
			Call UsageMsg
		End If
	    doD = 1

	Case "-v":
		If doV > -1 Then
			Call UsageMsg
		End If
		If doD = 1 Then 'Can't specify both -d and -v 
			Call UsageMsg 
		End If
 		doV = 1

	Case "-v3":
		If doV > -1 Then
			Call UsageMsg
		End If
		If doD = 1 Then 'Can't specify both -d and -v 
			Call UsageMsg 
		End If
 		doV = 3

	Case "-1":
		If ver > -1 Then
			Wscript.Echo "ver already set to " & ver
			Call UsageMsg
		End If	
		ver = 1

	Case "-2":
		If ver > -1 Then
			Wscript.Echo "ver already set to " & ver
			Call UsageMsg
		End If	
		ver = 2

	Case "-3":
		If ver > -1 Then
			Wscript.Echo "ver already set to " & ver
			Call UsageMsg
		End If	
		ver = 3

	Case "-n":
		If Os = "w" Then
			Wscript.Echo "OS already set to " & Os
			Call UsageMsg
		End If	
		Os = "n"

	Case "-w":
		If Os = "n" Then
	                Wscript.Echo "OS already set to " & Os
			Call UsageMsg
		End If	
		Os = "w"

	Case Else:
		Wscript.Echo "Illegal arg " & Wscript.Arguments(ArgCount)

		Call UsageMsg

  End Select

  ' Move pointer to next argument
  ArgCount = ArgCount + 1

Wend

rem **********************************************************************
rem * Check for required arguments and set any defaults
rem **********************************************************************
If Os = "z" Then
	Wscript.Echo "OS type error"
	Call UsageMsg
End If

If ver = -1 Then
	Wscript.Echo "iSCSI version error"
	Call UsageMsg
End If

If Os = "n" And doD > -1 Then
	Wscript.Echo "Cannot use Dynamic Disks on Windows NT"
	Call UsageMsg
End If

If ver = 3 And Os = "n" Then
	Wscript.Echo "Cannot use version 3 with Windows NT"
	Call UsageMsg
End If


rem **********************************************************************
rem * Determine if the iSCSI connections are established
rem **********************************************************************
Do
	retVal = WshShell.Run("chkiscsi.exe -" & ver & " -" & Os & " -l", 0, TRUE)

	If retVal = 0 Then
		WScript.Sleep 5000
	End If
Loop Until retVal <> 0
Wscript.Echo "Logged in!"

rem **********************************************************************
rem * Wait for StartDelay milliseconds for Windows to discover the devices
rem **********************************************************************
WScript.Sleep StartDelay


rem **********************************************************************
rem * Activate Dynamic Disks
rem **********************************************************************
If doD = 1 Then
	' Create command file to feed to the diskpart utility to have it
	' do a "list disk" command and output that to the actisci.txt
	' file for parsing
	Const ForReading = 1, ForWriting = 2
	Set diskPartCmds = fso.OpenTextFile(tempDir & "\dpartcmds.txt", ForWriting, True)
	diskPartCmds.WriteLine("list disk")
	diskPartCmds.Close
	retVal = WshShell.Run("%comspec% /c diskpart /s " & tempDir & "\dpartcmds.txt > " & tempDir & "\actiscsi.txt", 0, TRUE)
	If retVal <> 0 Then
		Wscript.Echo "diskpart returned error " & retVal 
	Else
		Dim i
		Dim listFound, diskStr
		i = 0
		listFound = 0

		Set f = fso.OpenTextFile(tempDir & "\actiscsi.txt", ForReading)

		' Parse the actiscsi.txt file and create a new dpartcmds.txt file
		' to online the dynamic disks.  Assumptions are made about
		' how the diskpart utility outputs the list of disks.  Beware
		' that this may change some day.
		While Not f.AtEndOfStream And listFound = 0
			f.Skip(2)
			If f.Read(1) = "-" Then
				listFound = 1 
			End If
   			f.SkipLine
		Wend

		Set diskPartCmds = fso.OpenTextFile(tempDir & "\dpartcmds.txt", ForWriting, True)
		While Not f.AtEndOfStream
			f.Skip(2)
			diskStr = f.Read(8)
   			f.Skip(33)
  			If  f.Read(1) = "*" Then
   				diskPartCmds.WriteLine "select " & diskStr
   				diskPartCmds.WriteLine "online"
   			End If
			f.SkipLine
		Wend
		diskPartCmds.Close
		f.Close

		' Send the command to diskpart to online all of the dynamic disks
		retVal = WshShell.Run("%comspec% /c diskpart /s " & tempDir & "\dpartcmds.txt", 0, TRUE)
	End If

End If


rem **********************************************************************
rem * Import Veritas VM (Version 3.0) disks groups
rem **********************************************************************
If doV = 1 Then
	tryCount = 4
	retVal = WshShell.Run("vxassist rescan", 0, TRUE)
	Wscript.Echo "vxassist rescan returned " & retVal 

	If retVal <> 1 Then ' Skip Veritas import if vxassist failed.
		' Loop until all Veritas disk groups have been imported.
		'
		' Find all the Veritas disk groups using "vxdg list".  Parse the output
		' of this command to find any deported disk groups.  Import these disk
		' groups using "vxdg import".

		Do
			retVal = WshShell.Run("%comspec% /c vxdg list > %temp%\actiscsi.txt", 0, TRUE)
			Wscript.Echo "vxdg list returned " & retVal 
			If retVal <> 0 Then
				iFail = 1
			Else
				Set f = fso.OpenTextFile(tempDir & "\actiscsi.txt", 1, False)
				If Err.Number = 0 Then
					iFail = 0
					Do While Not f.AtEndOfStream
						rdStr = f.ReadLine
						splitStr = Split(rdStr, "(", 2, 1)
						If UBound(splitStr) = 1 Then
							impPos = InStr(splitStr(0), unkStr)
							If impPos > 0 Then
								impPos = 0
							Else
								impPos = InStr(splitStr(1), impStr)
							End If
							If impPos > 0 Then
								dgName = Mid(splitStr(0),5,240)
								cluPos = InStr(splitStr(1), cluStr)
								If cluPos > 0 Then
									runStr = "vxdg import -s -g" & dgName
								Else
									runStr = "vxdg import -g" & dgName
								End If
								retVal = WshShell.Run(runStr, 0, TRUE)
								If retVal <> 0 Then
									iFail = 1
								End If
							End If
						End If
					Loop
					f.Close()
					Set f = fso.GetFile(tempDir & "\actiscsi.txt")
					f.Delete(True)
				Else
					iFail = 1
				End If
			End If

			tryCount = tryCount - 1

			If iFail = 1 Then
				WScript.Sleep 10000
			End If
		Loop Until iFail = 0 or tryCount <= 0

	Else
		Wscript.Echo "Import Veritas VM disk groups failed." 
	End If

End If



rem **********************************************************************
rem * Import Veritas VM (Version 3.1) disks groups
rem **********************************************************************
If doV = 3 Then
	tryCount = 4
	retVal = WshShell.Run("vxassist rescan", 0, TRUE)
	If retVal <> 1 Then ' Skip Veritas import if vxassist failed.
		' Loop until all Veritas disk groups have been imported.
		'
		' Find all the Veritas disk groups using "vxdg list".  Parse the output
		' of this command to find any deported disk groups.  Import these disk
		' groups using "vxdg import".
		Do
			retVal = WshShell.Run("%comspec% /c vxdg list > %temp%\actiscsi.txt", 0, TRUE)
			If retVal <> 0 Then
				iFail = 1
			Else
				Set f = fso.OpenTextFile(tempDir & "\actiscsi.txt", 1, False)
				If Err.Number = 0 Then
					iFail = 0
					cluPos = 0
					Do While Not f.AtEndOfStream
						rdStr = f.ReadLine
						splitStr = Split(rdStr, ":", 2, 1)
						If UBound(splitStr) = 1 Then
							impPos = InStr(splitStr(0), nameStr)
							If impPos > 0 Then
								runStr = Mid(splitStr(1),3,240)
								strLen = Len(runStr)
								dgName = Mid(runStr,1,strLen-1)
							End If
							impPos = InStr(splitStr(0), dgtypeStr)
							If impPos > 0 Then
								cluPos = InStr(splitStr(1), cluStr)
							End If
							impPos = InStr(splitStr(0), dgidStr)
							If impPos > 0 Then
								' holder in case I need it
								impPos = 0
							End If
							impPos = InStr(splitStr(0), statStr)
							If impPos > 0 Then
								impPos = InStr(splitStr(1), impStr)
							Else
								impPos = 0
							End If
							If impPos > 0 Then
								If cluPos > 0 Then
									runStr = "vxdg import -s -g" & dgName
									cluPos = 0
								Else
									runStr = "vxdg import -g" & dgName
								End If
								retVal = WshShell.Run(runStr, 0, TRUE)
								If retVal <> 0 Then
									iFail = 1
								End If
							End If
						End If
					Loop
					f.Close()
					Set f = fso.GetFile(tempDir & "\actiscsi.txt")
					f.Delete(True)
				Else
					iFail = 1
				End If
			End If
			tryCount = tryCount - 1
			If iFail = 1 Then
				WScript.Sleep 10000
			End If
		Loop Until iFail = 0 or tryCount <= 0
	Else
		Wscript.Echo "Import Veritas VM disk groups failed." 
	End If
End If



rem **********************************************************************
rem * Reestablish Disk Shares
rem **********************************************************************
If doS = 1 Then
	retVal = WshShell.Run("%comspec% /c net stop ""browser""", 0, TRUE)
	If Os = "w" Then
		retVal = WshShell.Run("%comspec% /c net stop ""dfs""", 0, TRUE)
	End If
	retVal = WshShell.Run("%comspec% /c net stop ""server""", 0, TRUE)
	retVal = WshShell.Run("%comspec% /c net start ""server""", 0, TRUE)
	If Os = "w" Then
		retVal = WshShell.Run("%comspec% /c net start ""dfs""", 0, TRUE)
	End If
	retVal = WshShell.Run("%comspec% /c net start ""browser""", 0, TRUE)

End If


rem **********************************************************************
rem * Start MS SQL Services
rem **********************************************************************
If doQ = 1 Then
		retVal = WshShell.Run("chkiscsi.exe -" & ver & " -" & Os & " -s ""MSSQL*""", 0, TRUE)
End If


rem **********************************************************************
rem * Start Oracle Services
rem **********************************************************************
If doO = 1 Then
		retVal = WshShell.Run("chkiscsi.exe -" & ver & " -" & Os & " -s ""OracleService*""", 0, TRUE)
		retVal = WshShell.Run("chkiscsi.exe -" & ver & " -" & Os & " -s ""Oracle*"" -x ""OracleConc*""", 0, TRUE)
		retVal = WshShell.Run("chkiscsi.exe -" & ver & " -" & Os & " -s ""OracleConc*""", 0, TRUE)

End If


rem **********************************************************************
rem * Start Exchange Services
rem **********************************************************************
If doE = 1 Then
		retVal = WshShell.Run("chkiscsi.exe -" & ver & " -" & Os & " -s ""Microsoft Exchange*""", 0, TRUE)
		retVal = WshShell.Run("chkiscsi.exe -" & ver & " -" & Os & " -s ""Simple Mail Transport Protocol*""", 0, TRUE)

End If


Wscript.Quit

rem **********************************************************************
rem * Displays usage message, then QUITS
rem **********************************************************************

Sub UsageMsg
	Wscript.Echo "Usage:  cscript actiscsi.vbs -<Version> -<Os> [-e] [-o] [-s] [-t time] [-<Disktype>]"
	Wscript.Echo "  Version:" 
	Wscript.Echo "      1    Version 1 iSCSI drivers" 
	Wscript.Echo "      2    Version 2 iSCSI drivers" 
	Wscript.Echo "      3    Version 3 iSCSI drivers" 
	Wscript.Echo "  Os:" 
	Wscript.Echo "      n    Windows NT" 
	Wscript.Echo "      w    Windows 2000" 
	Wscript.Echo "  -e    Start Exchange services" 
	Wscript.Echo "  -o    Start Oracle services" 
	Wscript.Echo "  -q    Start MS SQL services" 
	Wscript.Echo "  -s    Reestablish disk shares" 
    Wscript.Echo "  -t    Delay ""time"" in seconds to wait after detecting target logins"
    Wscript.Echo "        (1 to 120) seconds. Default = 20 seconds"
	Wscript.Echo "  Disktype:" 
	Wscript.Echo "      d    Activate Dynamic Disks" 
	Wscript.Echo "      v    Import Veritas VM (Version 3.0) disk groups"  
	Wscript.Echo "      v3   Import Veritas VM (Version 3.1) disk groups"  

	Wscript.Quit
End Sub

